import React, { useEffect, useState } from "react";
import {
  Platform,
  ToastAndroid,
  PermissionsAndroid,
  NativeModules
} from "react-native";

const { Alice: RNOnboarding } = NativeModules;


export const DocumentType = {
  IDCARD: "IDCARD",
  DRIVERLICENSE: "DRIVERLICENSE",
  RESIDENCEPERMIT: "RESIDENCEPERMIT",
  PASSPORT: "PASSPORT",
};

export const DocumentSide = {
  FRONT: "FRONT",
  BACK: "BACK",
};

function haveNonGrantedPermission(permissions) {
  return (
    Object.values(permissions).filter(response => response !== "granted")
      .length > 0
  );
}

function addComponentInfoConfig(config) {
  var pjson = require('./package.json');
  let componentInfoConfig = {
    "componentName": "onboarding-react-native", 
    "componentVersion": pjson.version
  }
  return {...config, ...componentInfoConfig}
}

function parseConfig(config) {
  if (Platform.OS === "android") {
    const { appearance, localization, ...rest } = config;
    return addComponentInfoConfig(rest);
  }
  return addComponentInfoConfig(config);
}

function handleAliceError(error, onCancel, onFailure) {
  if (error.message == "cancel") {
    onCancel(error.message);
  } else {
    onFailure(error.message);
  }
}

function usePermissions() {
  const [granted, setGranted] = useState(false);
  useEffect(() => {
    async function permissionsAsync() {
      // iOS permissions are checked natively
      if (Platform.OS === "android") {
        const response = await PermissionsAndroid.requestMultiple([
          PermissionsAndroid.PERMISSIONS.CAMERA,
          PermissionsAndroid.PERMISSIONS.WRITE_EXTERNAL_STORAGE
        ]);
        if (haveNonGrantedPermission(response)) {
          return ToastAndroid.show(
            "You must accept all permissions to be able to use Alice SDK",
            ToastAndroid.SHORT
          );
        }
      }
      setGranted(true);
    }
    permissionsAsync();
  });
  return granted;
}

export default ({ userToken, config, onSuccess, onFailure, onCancel }) => {
  const granted = usePermissions();
  useEffect(() => {
    async function aliceAsync() {
      try {
        const response = await RNOnboarding.runOnboarding(
          userToken,
          parseConfig(config)
        );
        onSuccess(response);
      } catch (error) {
        handleAliceError(error, onCancel, onFailure);
      }
    }

    if (granted) {
      aliceAsync();
    }
  }, [granted]);

  return null;
};


export function OnboardingWithSandbox({
  sandboxToken,
  email,
  config,
  onSuccess,
  onFailure,
  onCancel,
  firstName = "",
  lastName = ""
}) {
  const granted = usePermissions();
  useEffect(() => {
   async function aliceAsync() {
      try {
        const userToken = await RNOnboarding.logInWithSandbox(
          sandboxToken,
          email,
          firstName,
          lastName
        );
        const response = await RNOnboarding.runOnboarding(
          userToken,
          parseConfig(config)
        );
        onSuccess(response);
      } catch (error) {
        handleAliceError(error, onCancel, onFailure);
      }
    }

    if (granted) {
      aliceAsync();
    }
  }, [granted]);

  return null;
}

export function runOnboarding(userToken, config, onSuccess, onFailure, onCancel) {
    async function aliceAsync() {
        try {
            const response = await RNOnboarding.runOnboarding(
                userToken,
                parseConfig(config)
            );
            onSuccess(response);
        } catch (error) {
            handleAliceError(error, onCancel, onFailure);
        }
    }
    aliceAsync();
};

export function getUserTokenWithSandbox(
  sandboxToken,
  email,
  onSuccess,
  onFailure,
  onCancel,
  firstName = "",
  lastName = ""
) {
    async function aliceAsync() {
      try {
        const userToken = await RNOnboarding.logInWithSandbox(
          sandboxToken,
          email,
          firstName,
          lastName
        );
        onSuccess(userToken);
      } catch (error) {
        handleAliceError(error, onCancel, onFailure);
      }
    }
    aliceAsync();
}

export function GetUserTokenWithSandbox({
  sandboxToken,
  email,
  onSuccess,
  onFailure,
  onCancel,
  firstName = "",
  lastName = ""
}) {
  const granted = usePermissions();
  useEffect(() => {
    async function aliceAsync() {
      try {
        const userToken = await RNOnboarding.logInWithSandbox(
          sandboxToken,
          email,
          firstName,
          lastName
        );
        onSuccess(userToken);
      } catch (error) {
        handleAliceError(error, onCancel, onFailure);
      }
    }

    if (granted) {
      aliceAsync();
    }
  }, [granted]);

  return null;
}

export class OnboardingCommands {
  constructor(userToken) {
    this.userToken = userToken;
  }

  commandGetUserStatus(onSuccess, onError) {
      var userToken = this.userToken
       async function aliceAsync() {
          try {
             const status = await RNOnboarding.commandGetUserStatus(userToken);
                onSuccess(status);
          } catch (error) {
            handleAliceError(error, null, onError);
          }
        }
         aliceAsync();
    }

   commandGetDocumentsSupported(onSuccess, onError) {
       var userToken = this.userToken
       async function aliceAsync() {
          try {
             const status = await RNOnboarding.commandGetDocumentsSupported(userToken);
                onSuccess(status);
          } catch (error) {
            handleAliceError(error, null, onError);
          }
        }
         aliceAsync();
    }

    commandCreateDocument(type, issuingCountry ,onSuccess, onError) {
    var userToken = this.userToken
       async function aliceAsync() {
          try {
             const status = await RNOnboarding.commandCreateDocument(userToken, type, issuingCountry);
                onSuccess(status);
          } catch (error) {
            handleAliceError(error, null, onError);

          }

        }
         aliceAsync();
    }

   commandAddSelfie(onSuccess, onError, onCancel) {
       var userToken = this.userToken
       requestCameraPermission();
       async function aliceAsync() {
          try {
             const result = await RNOnboarding.commandAddSelfie(userToken);
                onSuccess(result);
          } catch (error) {
            handleAliceError(error, onCancel, onError);
          }
        }
         aliceAsync();
    }

   commandAddDocument(documentId, type, side, issuingCountry, onSuccess, onError, onCancel) {
       requestCameraPermission();
       var userToken = this.userToken
       async function aliceAsync() {
          try {
             const result = await RNOnboarding.commandAddDocument(userToken= userToken, documentId=documentId, type= type, side= side, issuingCountry= issuingCountry);
                onSuccess(result);
          } catch (error) {
            handleAliceError(error, onCancel, onError);
          }
        }
         aliceAsync();
    }

    commandAuthenticate(onSuccess, onError, onCancel) {
       var userToken = this.userToken
       requestCameraPermission();
       async function getUserStatusAsync() {
          try {
             const result = await RNOnboarding.commandGetUserStatus(userToken);
             if(JSON.parse(result).user.authorized) {
                   aliceAuthenticateAsync();
             } else {
                   onError("User no authorized");
             }
          } catch (error) {
            handleAliceError(error, onCancel, onError);
          }
        }
       async function aliceAuthenticateAsync() {
          try {
             const result = await RNOnboarding.commandAuthenticate(userToken);
             onSuccess(result);
          } catch (error) {
            handleAliceError(error, onCancel, onError);
          }
        }
        getUserStatusAsync();
    }


}


const requestCameraPermission = async () => {
  try {

    const granted = await PermissionsAndroid.request(
      PermissionsAndroid.PERMISSIONS.WRITE_EXTERNAL_STORAGE,
      {
        title: "Cool Photo App Camera Permission",
        message:
          "Cool Photo App needs access to your camera " +
          "so you can take awesome pictures.",
        buttonNeutral: "Ask Me Later",
        buttonNegative: "Cancel",
        buttonPositive: "OK"
      }
    );
    if (granted === PermissionsAndroid.RESULTS.GRANTED) {
      console.log("You can use the camera");
    } else {
      console.log("Camera permission denied");
    }
  } catch (err) {
    console.warn(err);
  }
};